/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef GLOBALINTERRUPTFLAG_H
#define GLOBALINTERRUPTFLAG_H

/**
 * Helper class for managing the global interrupt flag.
 */
class GlobalInterruptFlag
{
public:
	/**
	 * Sets the global interrupt flag, independent from previous state.
	 */
	static void set(void);

	/**
	 * Clears the global interrupt flag and returns the previous value of the SREG register.
	 * @result Value of SREG before clearing the global interrupt flag.
	 */
	static uint8_t clear(void);

	/**
	 * Returns true if global interrupt flag is set.
	 * @result True if global interrupt flag is set, false otherwise.
	 */
	static bool isSet(void);

	/**
	 * Restore old state of global interrupt flag using previously stored SREG value.
	 * @param sreg SREG value to restore global interrupt flag from
	 */
	static void restore(uint8_t sreg);
};

#endif
