/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef VIRTUALOUTPUTPORT_H
#define VIRTUALOUTPUTPORT_H

#include "outputport.h"
#include "outputpin.h"
#include "pinarray.h"

/**
 * Virtual 8 bit port consisting of 8 pins of type OutputPin. This is a convenience wrapper for PinArray implementing
 * the OutputPort interface.
 */
class VirtualOutputPort : public OutputPort
{
public:
	/**
	 * Constructor. Assigns pins to internal PinArray. The parameters intentionally don't have default values so
	 * that you have to make up your mind. See notes.
	 * @param pin0 Pin to be used for bit position 0.
	 * @param pin1 Pin to be used for bit position 1.
	 * @param pin2 Pin to be used for bit position 2.
	 * @param pin3 Pin to be used for bit position 3.
	 * @param pin4 Pin to be used for bit position 4.
	 * @param pin5 Pin to be used for bit position 5.
	 * @param pin6 Pin to be used for bit position 6.
	 * @param pin7 Pin to be used for bit position 7.
	 * @note You can specify null pointers for pins that should not be used (meaning bits that will be ignored
	 *       when writing).
	 * @note You can specify the same pin multiple times.
	 */
	VirtualOutputPort(OutputPin *pin0, OutputPin *pin1, OutputPin *pin2, OutputPin *pin3, OutputPin *pin4, OutputPin *pin5, OutputPin *pin6, OutputPin *pin7)
	{
		m_pinArray.setPin(0, pin0);
		m_pinArray.setPin(1, pin1);
		m_pinArray.setPin(2, pin2);
		m_pinArray.setPin(3, pin3);
		m_pinArray.setPin(4, pin4);
		m_pinArray.setPin(5, pin5);
		m_pinArray.setPin(6, pin6);
		m_pinArray.setPin(7, pin7);
	}

	/**
	 * Set value being represented by set of pins.
	 * @param value Value to be set.
	 * @param mask Mask with pins to manipulate according to value.
	 */
	virtual void setValue(uint8_t value, uint8_t mask = 0xff) override
	{
		m_pinArray.setValue(value, mask);
	}

private:
	PinArray<OutputPin, uint8_t, 8> m_pinArray;
};

#endif
