/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef INT2_H
#define INT2_H

#include "externalinterrupt.h"

/**
 * Implementation of ExternalInterrupt interface for the INT2 pin (PB2 in ATmega32A).
 * @note For controlling direction, pullup and pin state a separate instance of GpioPortPin is required.
 * @note INT2 supports trigger modes FALLING_EDGE and RISING_EDGE only. Other modes are implicitly
 *       mapped to FALLING_EDGE.
 */
class Int2 : public ExternalInterrupt
{
public:
	Int2(void);

	/**
	 * Sets the trigger mode.
	 * @param mode Trigger mode to set.
	 * @warning Although the enumeration contains all trigger modes, INT2 supports
	 *          FALLING_EDGE and RISING_EDGE only. All other modes are mapped to
	 *          FALLING_EDGE.
	 */
	virtual void setTriggerMode(TriggerMode mode) override;
	virtual void enableInterrupt(bool state) override;
	virtual void setInterruptHandler(InterruptHandler *handler) override;
};

#endif
