/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifndef USART0_H
#define USART0_H

#include "usart.h"

/**
 * Implements the Usart interface using the Usart0 hardware unit.
 */
class Usart0 : public Usart
{
public:
	Usart0(void);

	virtual void enable(bool state) override;
	virtual void configure(const Config &config) override;
	virtual void setData(uint8_t data) override;
	virtual uint8_t getData(void) override;
	virtual bool isReadable(void) override;
	virtual bool isWritable(void) override;
	virtual void setInterruptHandler(InterruptHandler *handler) override;
	virtual void enableDataRegisterEmptyInterrupt(bool state) override;
	virtual void enableRxCompleteInterrupt(bool state) override;

private:
	void setBaudRateSetting(const Config::BaudRateSetting &baudRateSetting);
	void setDataBits(Config::DataBits dataBits);
	void setParity(Config::Parity parity);
	void setStopBits(Config::StopBits stopBits);
};

#endif
