/*
 * Copyright (c) 2019, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "automaticinterruptlock.h"
#include "int2.h"

static volatile ExternalInterrupt::InterruptHandler *Int2_InterruptHandler;

ISR(INT2_vect)
{
	if (Int2_InterruptHandler)
	{
		const_cast<ExternalInterrupt::InterruptHandler*>(Int2_InterruptHandler)->onExternalInterrupt();
	}
}

Int2::Int2(void)
{
	// Safe defaults: falling edge trigger, no interrupts
	MCUCSR &= ~(1 << ISC2);
	GICR &= ~(1 << INT2);
	Int2_InterruptHandler = nullptr;
}

void Int2::setTriggerMode(TriggerMode mode)
{
	switch (mode)
	{
	case LOW_LEVEL:
		// not supported, fall-through to FALLING_EDGE
	case ANY_CHANGE:
		// not supported, fall-through to FALLING_EDGE
	case FALLING_EDGE:
		MCUCSR &= ~(1 << ISC2);
		break;
	case RISING_EDGE:
		MCUCSR |= (1 << ISC2);
		break;
	}
}

void Int2::enableInterrupt(bool state)
{
	if (state)
	{
		GICR |= (1 << INT2);
	}
	else
	{
		GICR &= ~(1 << INT2);
	}
}

void Int2::setInterruptHandler(InterruptHandler *handler)
{
	AutomaticInterruptLock lock;
	Int2_InterruptHandler = handler;
}
